from wnetalign.spectrum import Spectrum_1D
from wnetalign.aligner import WNetAligner as Solver
import numpy as np
import math

def test_matching():
    s1 = Spectrum_1D([0], [10])
    s2 = Spectrum_1D([1, 2], [5, 5])
    WN = Solver(s1, [s2], lambda x, y: np.linalg.norm(x - y, axis=0), 100, 10, 10000)
    WN.set_point([1])
    assert math.isclose(WN.total_cost(), 15.0)


def test_matching2():
    s1 = Spectrum_1D([0], [10])
    s2 = Spectrum_1D([1], [4])
    s3 = Spectrum_1D([2], [6])
    WN = Solver(
        s1, [s2, s3], lambda x, y: np.linalg.norm(x - y, axis=0), 100, 10, 10000
    )
    WN.set_point([1, 1])
    assert math.isclose(WN.total_cost(), 16.0)


def test_matching3():
    s1 = Spectrum_1D([0], [10])
    s2 = Spectrum_1D([1], [4])
    s3 = Spectrum_1D([200], [6])
    WN = Solver(
        s1, [s2, s3], lambda x, y: np.linalg.norm(x - y, axis=0), 10, 10, 100
    )
    WN.set_point([1, 1])
    WN.print_diagnostics()
    print(WN.flows())
    print("aaa",WN.total_cost())
    assert math.isclose(WN.total_cost(), 64.0)

if __name__ == "__main__":
    test_matching()
    test_matching2()
    test_matching3()
    print("Everything passed")



from wnet import Distribution, Distribution_1D, WassersteinNetwork
from wnetalign import WNetAligner
from wnet.distances import wrap_distance_function
import numpy as np


def test_1d():
    E = Distribution_1D([1], [1])
    T = Distribution_1D([2], [1])
    solver = WNetAligner(E, [T], lambda x, y: np.linalg.norm(x - y, axis=0), 10, 10, 100)
    #solver = WassersteinSolver(E, [T], [SimpleTrash(10)])
    solver.set_point([1])
    assert solver.total_cost() == 1


def test_2d():
    s1_pos = np.array([[0, 1, 0], [0, 0, 1]])
    s1_int = np.array([1, 1, 1])
    s1 = Distribution(s1_pos, s1_int)
    s2_pos = np.array([[1, 1, 0], [1, 0, 1]])
    s2_int = np.array([1, 1, 1])
    s2 = Distribution(s2_pos, s2_int)
    solver = WNetAligner(s1, [s2], lambda x, y: np.linalg.norm(x - y, axis=0), 1000000, 1000000, 1000)
    solver.set_point([1])
    # print(solver.run())
    assert solver.total_cost() == 1.414

    # new algo
    DG = WassersteinNetwork(
        s1, [s2], wrap_distance_function(lambda x, y: 1000 * np.linalg.norm(x - y, axis=0)), 5000
    )
    DG.add_simple_trash(1000000)
    DG.build()

    DG.solve()
    assert DG.total_cost() == 1414



if __name__ == "__main__":
    test_1d()
    test_2d()
    print("Everything passed")



from wnet import Distribution, WassersteinNetwork
from collections import namedtuple
import numpy as np


class WNetAligner:
    def __init__(
        self,
        empirical_spectrum,
        theoretical_spectra,
        distance_function,
        max_distance,
        trash_cost,
        scale_factor=None,
    ):
        assert isinstance(empirical_spectrum, Distribution)
        assert isinstance(theoretical_spectra, list)
        assert all(isinstance(t, Distribution) for t in theoretical_spectra)
        assert callable(distance_function)
        assert isinstance(max_distance, (int, float))
        assert isinstance(trash_cost, (int, float))
        assert scale_factor is None or isinstance(scale_factor, (int, float))

        if scale_factor is None:
            ALMOST_MAXINT = 2**60
            empirical_sum_intensity = empirical_spectrum.sum_intensities
            theoretical_sum_intensity = sum(
                t.sum_intensities for t in theoretical_spectra
            )
            max_sum_intensity = max(
                empirical_sum_intensity, theoretical_sum_intensity
            )
            scale_factor = np.sqrt(ALMOST_MAXINT / (max_sum_intensity * trash_cost))
            assert scale_factor > 0, "Can't auto-compute a sensible scale factor. You might have some luck with setting it manually, but it probably means something about your data or trash_cost is off."

        self.scale_factor = scale_factor
        self.empirical_spectrum = empirical_spectrum.scaled(scale_factor)
        self.theoretical_spectra = [t.scaled(scale_factor) for t in theoretical_spectra]

        def wrapped_dist(p, y):
            i = p.index
            x = p.positions[:, i : i + 1]
            return distance_function(x[: np.newaxis], y) * scale_factor

        self.graph = WassersteinNetwork(
            self.empirical_spectrum,
            self.theoretical_spectra,
            wrapped_dist,
            int(max_distance * scale_factor),
        )
        self.graph.add_simple_trash(int(trash_cost * scale_factor))
        self.graph.build()
        self.point = None

    def set_point(self, point):
        self.point = point
        self.graph.solve(point)

    def total_cost(self):
        return self.graph.total_cost() / self.scale_factor / self.scale_factor

    def print(self):
        print(str(self.graph))

    def flows(self):
        result = []
        for i in range(len(self.theoretical_spectra)):
            empirical_peak_idx, theoretical_peak_idx, flow = (
                self.graph.flows_for_target(i)
            )
            result.append(
                namedtuple(
                    "Flow", ["empirical_peak_idx", "theoretical_peak_idx", "flow"]
                )(empirical_peak_idx, theoretical_peak_idx, flow / self.scale_factor)
            )
        return result

    def no_subgraphs(self):
        return self.graph.no_subgraphs()

    def print_diagnostics(self, subgraphs_too=False):
        print("Diagnostics:")
        print("No subgraphs:", self.graph.no_subgraphs())
        print("No empirical nodes:", self.graph.count_empirical_nodes())
        print("No theoretical nodes:", self.graph.count_theoretical_nodes())
        print("Matching density:", self.graph.matching_density())
        print("Scale factor:", self.scale_factor, f" log10: {np.log10(self.scale_factor)}")
        print("Total cost:", self.graph.total_cost())
        if not subgraphs_too:
            return
        for ii in range(self.graph.no_subgraphs()):
            s = self.graph.get_subgraph(ii)
            print("Subgraph", ii, ":")
            print("  No. empirical nodes:", s.count_empirical_nodes())
            print("  No. theoretical nodes:", s.count_theoretical_nodes())
            print("  Cost:", s.total_cost())
            print("  Matching density:", s.matching_density())
            print("  Theoretical spectra involved:", s.theoretical_spectra_involved())



import numpy as np


from wnet.distribution import Distribution
from wnetalign.aligner import WNetAligner as Solver


def test_flows():
    S1 = Distribution(np.array([[1, 2, 30]]), np.array([1, 4, 3]))
    S2 = Distribution(np.array([[1, 4, 30, 31]]), np.array([5, 1, 1, 1]))

    dist_fun = lambda x, y: np.linalg.norm(x - y, axis=0)
    trash_cost = 10
    max_distance = 100
    solver = Solver(
        empirical_spectrum=S1,
        theoretical_spectra=[S2],
        distance_function=dist_fun,
        max_distance=max_distance,
        trash_cost=trash_cost,
        scale_factor=None,
    )

    solver.set_point([1])

    solver.print_diagnostics()

    print("Flows:")
    for flow in solver.flows():
        print(flow)

    #DGW = DecompositableGraphWrapper(solver.graph)
    #SG = list(DGW.get_subgraphs())[0]
    #print(SG.as_nx_graph())
    #SG.show()


if __name__ == "__main__":
    test_flows()
    print("Everything passed")
